//
// Copyright 2011 eviware software ab
//
// Licensed under the EUPL, Version 1.1 or - as soon they will be approved by the European Commission - subsequent
// versions of the EUPL (the "Licence");
// You may not use this work except in compliance with the Licence.
// You may obtain a copy of the Licence at:
//
// http://ec.europa.eu/idabc/eupl5
//
// Unless required by applicable law or agreed to in writing, software distributed under the Licence is
// distributed on an "AS IS" basis, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
// express or implied. See the Licence for the specific language governing permissions and limitations
// under the Licence.
//

/**
 * Post selected fields of incoming messages as Twitter status updates. 
 * 
 * @help http://www.loadui.org/Output/twitter-component.html
 * @name Twitter
 * @category output
 * @dependency oauth.signpost:signpost-core:1.2.1.1
 */

import java.net.HttpURLConnection
import java.net.URL

import java.awt.Desktop
import java.net.URI

import oauth.signpost.OAuth
import oauth.signpost.OAuthConsumer
import oauth.signpost.OAuthProvider
import oauth.signpost.basic.DefaultOAuthConsumer
import oauth.signpost.basic.DefaultOAuthProvider
 
import com.eviware.loadui.api.events.ActionEvent
import com.eviware.loadui.util.layout.DelayedFormattedString

createProperty ('appendedHashTag', String, '#loaduiTwitterComponent')
createProperty ('minutesBetweenTweets', Integer, 5)
accessToken = createProperty ('_accessToken', String)
tokenSecret = createProperty ('_tokenSecret', String)
username = createProperty ('_userName', String)
pin = createProperty ('_pin', String)
isConfigured = createProperty ('_isConfigured', Boolean, false)
isConnecting = createProperty ('_isConnecting', Boolean, false)

consumer = new DefaultOAuthConsumer("q2GuoDXLrPKXvduLApGHg","NnxuiDkdV6CiAmhubdQkkP34QtrFEIfoqrvV3PsM");
provider = new DefaultOAuthProvider("http://twitter.com/oauth/request_token","http://twitter.com/oauth/access_token", "http://twitter.com/oauth/authorize");

inputFields = []
tweetsMade = 0
tweetsFailed = 0
lastTimeTweeted = 0

tweetsMadeDisplay = new DelayedFormattedString( '%d', 500, value { tweetsMade } )
tweetsFailedDisplay = new DelayedFormattedString( '%s', 500, value { tweetsFailed } )
usernameDisplay = new DelayedFormattedString( '%s', 1000, value { username.value } )
statusDisplay = new DelayedFormattedString( '%s', 1000, value {
		Integer timeUntilReady = calcTimeUntilReady()
		if( timeUntilReady <= 0 )
			"Ready"
		else
			"Blocked ($timeUntilReady)"
	} 
)

output = { message ->
	if( isConfigured.value )
		tweet( message, false )
}

onRelease = {
	tweetsMadeDisplay.release()
	tweetsFailedDisplay.release()
	usernameDisplay.release()
}

onConnect = { outgoing, incoming ->
	if( incoming == inputTerminal ){
		redraw()
	}
}

onDisconnect = { outgoing, incoming ->
	if( incoming == inputTerminal ){
		redraw()
	}
}

onSignature = { outgoing, signature ->
	redraw()
}

def Integer calcTimeUntilReady()
{
	return (minutesBetweenTweets.value * 60000 - (System.currentTimeMillis() - lastTimeTweeted))/1000
}

addEventListener( ActionEvent ) { event ->
	if ( event.key == "RESET" ) {
		tweetsMade = 0
		tweetsFailed = 0
		lastTimeTweeted = 0
	}
}

def tweet( terminalMessage, isTesting )
{
	if( calcTimeUntilReady() > 0 )
	{
		tweetsFailed++
		return
	}

	if( isTesting )
	{
		twitterMessage = "Testing $appendedHashTag.value"
	}
	else
	{
		twitterMessage = inputFields.findAll{ it.value }.collect{ "$it.key:" + terminalMessage[it.key] }.join(", ")

		if( twitterMessage.size() > 140 )
			twitterMessage = twitterMessage[0..140]
		else if( "$twitterMessage $appendedHashTag.value".size()  <= 140 )
		{
			twitterMessage += " $appendedHashTag.value"
		}
	}
	
	try
	{
		consumer.setTokenWithSecret( accessToken.value, tokenSecret.value )
		
		URL url = new URL("http://api.twitter.com/1/statuses/update.json?status=" + URLEncoder.encode( twitterMessage ) )
		HttpURLConnection request = (HttpURLConnection) url.openConnection()
		request.setRequestMethod("POST")

		consumer.sign(request)
		request.connect()
		
		statusCode = request.getResponseCode()
		assert( statusCode == 200 )

	} catch (e) {
		println e
		tweetsFailed++
		return
	}
	tweetsMade++
	lastTimeTweeted = System.currentTimeMillis()
}

// layout
def redraw()
{
	for( field in inputFields )
		deleteProperty( field.key )
	inputFields = []
	for( conn in inputTerminal.connections ) {
		for( key in conn.outputTerminal.messageSignature.keySet() )
			inputFields.add( createProperty( key, Boolean, false ) )
	}
		
	if ( isConfigured.value )
	{
		layout
		{ 
			box( widget:'display', constraints: 'center, growx' ) {
				node( label: 'Username', fString:usernameDisplay )
				node( label: 'Status', fString:statusDisplay, constraints: 'w 64!' )
				node( label: 'Tweets', fString:tweetsMadeDisplay )
				node( label: 'Failures', fString:tweetsFailedDisplay )
				
			}
			separator( vertical: false )
			box( constraints: 'center' )
			{
				action(
					constraints: 'center, bottom',
					label:'Test',
					action:
					{
						tweet( [], true )
					}
				)
				action(
					constraints: 'center, bottom',
					label:'Change account',
					action:
					{
						isConfigured.value = false
						redraw()
					}
				)
			}
			if( inputFields.size() > 0 )
			{
				separator( vertical: false )
				box() {
					i = 0
					for( field in inputFields )
					{
						i++
						if( i < inputFields.size() )
							property( property: field, label: field.key, constraints: 'growx, w 150!, wrap 10px' )
						else
							property( property: field, label: field.key, constraints: 'growx, w 150!' )
					}
				}
			}
		}
	} else
	{
		layout {
			connectButtonConstraints = 'align center'
			if( isConnecting.value ) connectButtonConstraints += ', gap 0 0 0 10'
			action(
				label:'Connect with Twitter',
				constraints: connectButtonConstraints,
				action:
				{


					String authUrl = provider.retrieveRequestToken(consumer, OAuth.OUT_OF_BAND);

					java.net.URI uri = new java.net.URI( authUrl )
					java.awt.Desktop.getDesktop().browse( uri )
					isConnecting.value = true
					redraw()
				}
			)
			if( isConnecting.value )
			{
				separator( vertical:false )
				box(  )
				{
					property( property:pin, label:'Confirmation PIN' )
					action(
						label:'Ok',
						constraints: 'center, bottom',
						action:
						{
							provider.retrieveAccessToken(consumer, pin.value);
							accessToken.value = consumer.getToken()
							username.value = provider.getResponseParameter("screen_name")
							tokenSecret.value = consumer.getTokenSecret()
							isConnecting.value = false
							isConfigured.value = true
							redraw()
						}
					)
				}
			}
		}
	}
}

redraw()

compactLayout {
	box( widget:'display' ) {
		node( label: 'Status', fString:statusDisplay, constraints: 'w 64!' )
		node( label: 'Tweets', fString:tweetsMadeDisplay )
		node( label: 'Failures', fString:tweetsFailedDisplay )
	}
}
